// const fsPromise = require("fs/promises");
const FormData = require('form-data');
const crypto = require('crypto');

const IDGEN_URI = require('../config.js').IDGEN_URI
const Ks3Util = require('../utils/ksUtil').Ks3Util
const KS3_CONFIG = require('../constant').KS3_CONFIG
const req = require("../utils/request").httpReq;


const _get_obj_id = async (ctx, next) => {
  var url = `${IDGEN_URI}/Ksy/ksyun/buildObectid`;
  var opts = {
    url: url,
    qs: {
      appid: 'merchant-b',
    },
    method: 'GET',
  }
  return await req(ctx, opts);
}

const _get_bucket = async (ctx, next) => {
  var url = `${IDGEN_URI}/Ksy/ksyun/getBucket`;
  var opts = {
    url: url,
    qs: {
      appid: 'merchant-b',
    },
    method: 'GET',
  }
  // console.log(await req(ctx, opts))
  return await req(ctx, opts)
}

const _get_token = async (ctx, next) => {
  var url = `${IDGEN_URI}/Ksy/ksyun/getToken`;
  var opts = {
    url: url,
    data: {
      appid: 'merchant-b',
    },
    method: 'POST',
  }
  return await req(ctx, opts)
}


exports.get_obj_id = async (ctx, next) => {
  ctx.body = _get_obj_id(ctx, next)
}


exports.get_bucket = async (ctx, next) => {
  ctx.body = await _get_bucket(ctx, next)
}

exports.get_token = async (ctx, next) => {
  ctx.body = await _get_token(ctx, next)
}

exports.get_ks3_config = async (ctx, next) => {
  console.log(8888, await _get_bucket(ctx, next))
  const { result: { bucket } } = JSON.parse(await _get_bucket(ctx, next))
  const { result: { objectId } } = JSON.parse(await _get_obj_id(ctx, next))

  console.log(888834333, bucket, objectId)

  const getExpires = (seconds) => {
    return Math.round(new Date().getTime() / 1000) + seconds;
  };

  const policy = {
    "expiration": new Date(getExpires(3600) * 1000).toISOString(), //一小时后
    "conditions": [
      ["eq", "$bucket", bucket],
      ["starts-with", "$acl", "public-read"],
      // ["starts-with", "$key", ""],
      // ["starts-with", "$name", ""],   //表单中传了name字段，也需要加到policy中
      // ["starts-with", "$x-kss-meta-custom-param1",""],
      // ["starts-with", "$x-kss-newfilename-in-body",""],//必须只包含小写字符
      // ["starts-with", "$Cache-Control",""],
      // ["starts-with", "$Expires", ""],
      // ["starts-with", "$Content-Disposition", ""],
      // ["starts-with", "$Content-Type",""],
      // ["starts-with", "$Content-Encoding",""]
    ]
  }
  const stringToSign = Ks3Util.Base64.encode(JSON.stringify(policy))


  console.log('stringToSign:::', stringToSign)
  const signature = Ks3Util.b64_hmac_sha1(KS3_CONFIG.SK, stringToSign)
  // const signature = crypto.createHmac('sha1', KS3_CONFIG.SK).update(stringToSign).digest().toString('base64');
  console.log("signature::::::", signature)

  ctx.body = {
    bucket,
    objectId,
    policy: stringToSign,
    signature,
  }
}

exports.upload_ks3_image = async (ctx, next) => {

  // console.log('upload_ks3_image::::::', ctx.request.files)
  const file = ctx.request.files && ctx.request.files.file;
  console.log(888899, await _get_bucket(ctx, next))
  const { result: { bucket } } = JSON.parse(await _get_bucket(ctx, next))
  const { result: { objectId } } = JSON.parse(await _get_obj_id(ctx, next))
  console.log(3334444, file.type)

  const getExpires = (seconds) => {
    return Math.round(new Date().getTime() / 1000) + seconds;
  };

  const acl = 'public-read';

  const policy = {
    "expiration": new Date(getExpires(3600) * 1000).toISOString(), //一小时后
    "conditions": [
      ["eq", "$bucket", bucket],
      ["starts-with", "$acl", "public-read"],
    ]
  }
  const stringToSign = Ks3Util.Base64.encode(JSON.stringify(policy))

  console.log('stringToSign:::', stringToSign)
  const signature = Ks3Util.b64_hmac_sha1(KS3_CONFIG.SK, stringToSign)
  // const reader = fs.createReadStream(file.path);

  // const reader = await fsPromise.readFile(file.path);
  const reader = file;

  const formData = new FormData()
  // formData.append('acl', acl)
  // formData.append('key', objectId)
  // formData.append('signature', signature)
  // formData.append('KSSAccessKeyId', KS3_CONFIG.AK)
  // formData.append('policy', stringToSign)
  // formData.append('bucket_name', stringToSign)
  formData.append('file', reader)

  const attrAcl = `x-${KS3_CONFIG.prefix}-acl`;
  const date = new Date().toGMTString();
  console.log('!!!!!', date)

  const headersForSign = {
    // ...formData.getHeaders(),
    // 'content-length': formData.getLengthSync(),
    'x-kss-date': date,
    [attrAcl]: acl
  }

  const headerSignature = Ks3Util.generateToken(KS3_CONFIG.SK, bucket, objectId, 'PUT', file.type, headersForSign, date)

  const hmac = crypto.createHmac('sha256', KS3_CONFIG.SK);

  hmac.update(headerSignature);
  console.log(hmac.digest('hex'));

  console.log('headerSignature::::', headerSignature)
  const headers = {
    ...headersForSign,
    'Authorization': `KSS ${KS3_CONFIG.AK}:${headerSignature}`
  }

  console.log('before send::::::', headers)
  var opts = {
    url: `http://${KS3_CONFIG.DOMAIN}/${bucket}/${objectId}`,
    method: 'PUT',
    headers,
    data: file
  }
  console.log(3333333333, formData.getLengthSync());
  const res = await req(ctx, opts);
  console.log('sever ks3 image::::::', res)
  ctx.body = res;

  // const url = `http://${KS3_CONFIG.DOMAIN}/${bucket}/${objectId}`
  // var xhr = new XMLHttpRequest();
  // xhr.open('PUT', url, true);

  // var headers = {};
  // var acl = 'public-read';
  // if (acl == 'private' || acl == 'public-read') {
  //     var attr_Acl = 'x-' + KS3_CONFIG.prefix + '-acl';
  //     xhr.setRequestHeader(attr_Acl, acl);
  //     headers[attr_Acl] = acl;
  // }
  // var signature = Ks3Util.generateToken(KS3_CONFIG.SK, bucket, objectId, 'PUT', file.type, headers, '');

  // xhr.onreadystatechange = function() {
  //     if (xhr.readyState == 4) {
  //         if(xhr.status >= 200 && xhr.status < 300 || xhr.status == 304){
  //             cb(null);
  //         }else if(xhr.status === 413 || xhr.status === 415) {
  //             // var errMsg = Ks3.xmlToJson(xhr.responseXML)['Error']['Message'];
  //             // cb({"msg":errMsg});
  //             console.log('image::::', xhr.responseXML);
  //         }else {
  //             console.log('status: ' + xhr.status);
  //             cb({"msg":"request failed"});
  //         }
  //     }
  // };
  // // xhr.upload.addEventListener("progress", params.ProgressListener, false);

  // xhr.setRequestHeader('Authorization','KSS ' + KS3_CONFIG.AK + ':' + signature );
  // xhr.send(file);
}